@@ -44,7 +44,7 @@ def response(status_code=200, content='', headers=None, reason=None, elapsed=0,
     return res
 
 
-def fake_resolve_authconfig(authconfig, registry=None):
+def fake_resolve_authconfig(authconfig, registry=None, *args, **kwargs):
     return None
 
 
@@ -128,34 +128,27 @@ class DockerApiTest(BaseAPIClientTest):
         with pytest.raises(docker.errors.DockerException) as excinfo:
             APIClient(version=1.12)
 
-        self.assertEqual(
-            str(excinfo.value),
-            'Version parameter must be a string or None. Found float'
-        )
+        assert str(
+            excinfo.value
+        ) == 'Version parameter must be a string or None. Found float'
 
     def test_url_valid_resource(self):
         url = self.client._url('/hello/{0}/world', 'somename')
-        self.assertEqual(
-            url, '{0}{1}'.format(url_prefix, 'hello/somename/world')
-        )
+        assert url == '{0}{1}'.format(url_prefix, 'hello/somename/world')
 
         url = self.client._url(
             '/hello/{0}/world/{1}', 'somename', 'someothername'
         )
-        self.assertEqual(
-            url,
-            '{0}{1}'.format(url_prefix, 'hello/somename/world/someothername')
+        assert url == '{0}{1}'.format(
+            url_prefix, 'hello/somename/world/someothername'
         )
 
         url = self.client._url('/hello/{0}/world', 'some?name')
-        self.assertEqual(
-            url, '{0}{1}'.format(url_prefix, 'hello/some%3Fname/world')
-        )
+        assert url == '{0}{1}'.format(url_prefix, 'hello/some%3Fname/world')
 
         url = self.client._url("/images/{0}/push", "localhost:5000/image")
-        self.assertEqual(
-            url,
-            '{0}{1}'.format(url_prefix, 'images/localhost:5000/image/push')
+        assert url == '{0}{1}'.format(
+            url_prefix, 'images/localhost:5000/image/push'
         )
 
     def test_url_invalid_resource(self):
@@ -164,15 +157,13 @@ class DockerApiTest(BaseAPIClientTest):
 
     def test_url_no_resource(self):
         url = self.client._url('/simple')
-        self.assertEqual(url, '{0}{1}'.format(url_prefix, 'simple'))
+        assert url == '{0}{1}'.format(url_prefix, 'simple')
 
     def test_url_unversioned_api(self):
         url = self.client._url(
             '/hello/{0}/world', 'somename', versioned_api=False
         )
-        self.assertEqual(
-            url, '{0}{1}'.format(url_base, 'hello/somename/world')
-        )
+        assert url == '{0}{1}'.format(url_base, 'hello/somename/world')
 
     def test_version(self):
         self.client.version()
@@ -194,13 +185,13 @@ class DockerApiTest(BaseAPIClientTest):
 
     def test_retrieve_server_version(self):
         client = APIClient(version="auto")
-        self.assertTrue(isinstance(client._version, six.string_types))
-        self.assertFalse(client._version == "auto")
+        assert isinstance(client._version, six.string_types)
+        assert not (client._version == "auto")
         client.close()
 
     def test_auto_retrieve_server_version(self):
         version = self.client._retrieve_server_version()
-        self.assertTrue(isinstance(version, six.string_types))
+        assert isinstance(version, six.string_types)
 
     def test_info(self):
         self.client.info()
@@ -221,6 +212,24 @@ class DockerApiTest(BaseAPIClientTest):
             timeout=DEFAULT_TIMEOUT_SECONDS
         )
 
+    def test_login(self):
+        self.client.login('sakuya', 'izayoi')
+        args = fake_request.call_args
+        assert args[0][0] == 'POST'
+        assert args[0][1] == url_prefix + 'auth'
+        assert json.loads(args[1]['data']) == {
+            'username': 'sakuya', 'password': 'izayoi'
+        }
+        assert args[1]['headers'] == {'Content-Type': 'application/json'}
+        assert self.client._auth_configs['auths'] == {
+            'docker.io': {
+                'email': None,
+                'password': 'izayoi',
+                'username': 'sakuya',
+                'serveraddress': None,
+            }
+        }
+
     def test_events(self):
         self.client.events()
 
@@ -313,11 +322,10 @@ class DockerApiTest(BaseAPIClientTest):
     def test_create_host_config_secopt(self):
         security_opt = ['apparmor:test_profile']
         result = self.client.create_host_config(security_opt=security_opt)
-        self.assertIn('SecurityOpt', result)
-        self.assertEqual(result['SecurityOpt'], security_opt)
-        self.assertRaises(
-            TypeError, self.client.create_host_config, security_opt='wrong'
-        )
+        assert 'SecurityOpt' in result
+        assert result['SecurityOpt'] == security_opt
+        with pytest.raises(TypeError):
+            self.client.create_host_config(security_opt='wrong')
 
     def test_stream_helper_decoding(self):
         status_code, content = fake_api.fake_responses[url_prefix + 'events']()
@@ -335,29 +343,29 @@ class DockerApiTest(BaseAPIClientTest):
         raw_resp._fp.seek(0)
         resp = response(status_code=status_code, content=content, raw=raw_resp)
         result = next(self.client._stream_helper(resp))
-        self.assertEqual(result, content_str)
+        assert result == content_str
 
         # pass `decode=True` to the helper
         raw_resp._fp.seek(0)
         resp = response(status_code=status_code, content=content, raw=raw_resp)
         result = next(self.client._stream_helper(resp, decode=True))
-        self.assertEqual(result, content)
+        assert result == content
 
         # non-chunked response, pass `decode=False` to the helper
         setattr(raw_resp._fp, 'chunked', False)
         raw_resp._fp.seek(0)
         resp = response(status_code=status_code, content=content, raw=raw_resp)
         result = next(self.client._stream_helper(resp))
-        self.assertEqual(result, content_str.decode('utf-8'))
+        assert result == content_str.decode('utf-8')
 
         # non-chunked response, pass `decode=True` to the helper
         raw_resp._fp.seek(0)
         resp = response(status_code=status_code, content=content, raw=raw_resp)
         result = next(self.client._stream_helper(resp, decode=True))
-        self.assertEqual(result, content)
+        assert result == content
 
 
-class StreamTest(unittest.TestCase):
+class UnixSocketStreamTest(unittest.TestCase):
     def setUp(self):
         socket_dir = tempfile.mkdtemp()
         self.build_context = tempfile.mkdtemp()
@@ -442,21 +450,73 @@ class StreamTest(unittest.TestCase):
             b'\r\n'
         ) + b'\r\n'.join(lines)
 
-        with APIClient(base_url="http+unix://" + self.socket_file) \
-                as client:
+        with APIClient(base_url="http+unix://" + self.socket_file) as client:
             for i in range(5):
                 try:
                     stream = client.build(
                         path=self.build_context,
-                        stream=True
                     )
                     break
                 except requests.ConnectionError as e:
                     if i == 4:
                         raise e
 
-            self.assertEqual(list(stream), [
-                str(i).encode() for i in range(50)])
+            assert list(stream) == [
+                str(i).encode() for i in range(50)
+            ]
+
+
+class TCPSocketStreamTest(unittest.TestCase):
+    text_data = b'''
+    Now, those children out there, they're jumping through the
+    flames in the hope that the god of the fire will make them fruitful.
+    Really, you can't blame them. After all, what girl would not prefer the
+    child of a god to that of some acne-scarred artisan?
+    '''
+
+    def setUp(self):
+
+        self.server = six.moves.socketserver.ThreadingTCPServer(
+            ('', 0), self.get_handler_class()
+        )
+        self.thread = threading.Thread(target=self.server.serve_forever)
+        self.thread.setDaemon(True)
+        self.thread.start()
+        self.address = 'http://{}:{}'.format(
+            socket.gethostname(), self.server.server_address[1]
+        )
+
+    def tearDown(self):
+        self.server.shutdown()
+        self.server.server_close()
+        self.thread.join()
+
+    def get_handler_class(self):
+        text_data = self.text_data
+
+        class Handler(six.moves.BaseHTTPServer.BaseHTTPRequestHandler, object):
+            def do_POST(self):
+                self.send_response(101)
+                self.send_header(
+                    'Content-Type', 'application/vnd.docker.raw-stream'
+                )
+                self.send_header('Connection', 'Upgrade')
+                self.send_header('Upgrade', 'tcp')
+                self.end_headers()
+                self.wfile.flush()
+                time.sleep(0.2)
+                self.wfile.write(text_data)
+                self.wfile.flush()
+
+        return Handler
+
+    def test_read_from_socket(self):
+        with APIClient(base_url=self.address) as client:
+            resp = client._post(client._url('/dummy'), stream=True)
+            data = client._read_from_socket(resp, stream=True, tty=True)
+            results = b''.join(data)
+
+        assert results == self.text_data
 
 
 class UserAgentTest(unittest.TestCase):
@@ -475,18 +535,18 @@ class UserAgentTest(unittest.TestCase):
         client = APIClient()
         client.version()
 
-        self.assertEqual(self.mock_send.call_count, 1)
+        assert self.mock_send.call_count == 1
         headers = self.mock_send.call_args[0][0].headers
         expected = 'docker-sdk-python/%s' % docker.__version__
-        self.assertEqual(headers['User-Agent'], expected)
+        assert headers['User-Agent'] == expected
 
     def test_custom_user_agent(self):
         client = APIClient(user_agent='foo/bar')
         client.version()
 
-        self.assertEqual(self.mock_send.call_count, 1)
+        assert self.mock_send.call_count == 1
         headers = self.mock_send.call_args[0][0].headers
-        self.assertEqual(headers['User-Agent'], 'foo/bar')
+        assert headers['User-Agent'] == 'foo/bar'
 
 
 class DisableSocketTest(unittest.TestCase):
@@ -509,7 +569,7 @@ class DisableSocketTest(unittest.TestCase):
 
         self.client._disable_socket_timeout(socket)
 
-        self.assertEqual(socket.timeout, None)
+        assert socket.timeout is None
 
     def test_disable_socket_timeout2(self):
         """Test that the timeouts are disabled on a generic socket object
@@ -519,8 +579,8 @@ class DisableSocketTest(unittest.TestCase):
 
         self.client._disable_socket_timeout(socket)
 
-        self.assertEqual(socket.timeout, None)
-        self.assertEqual(socket._sock.timeout, None)
+        assert socket.timeout is None
+        assert socket._sock.timeout is None
 
     def test_disable_socket_timout_non_blocking(self):
         """Test that a non-blocking socket does not get set to blocking."""
@@ -529,5 +589,5 @@ class DisableSocketTest(unittest.TestCase):
 
         self.client._disable_socket_timeout(socket)
 
-        self.assertEqual(socket.timeout, None)
-        self.assertEqual(socket._sock.timeout, 0.0)
+        assert socket.timeout is None
+        assert socket._sock.timeout == 0.0
