@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 
 import unittest
-import warnings
 
 import pytest
 
 from docker.constants import DEFAULT_DOCKER_API_VERSION
 from docker.errors import InvalidArgument, InvalidVersion
 from docker.types import (
-    ContainerConfig, ContainerSpec, EndpointConfig, HostConfig, IPAMConfig,
+    ContainerSpec, EndpointConfig, HostConfig, IPAMConfig,
     IPAMPool, LogConfig, Mount, ServiceMode, Ulimit,
 )
+from docker.types.services import convert_service_ports
 
 try:
     from unittest import mock
@@ -23,88 +23,75 @@ def create_host_config(*args, **kwargs):
 
 
 class HostConfigTest(unittest.TestCase):
-    def test_create_host_config_no_options(self):
-        config = create_host_config(version='1.19')
-        self.assertFalse('NetworkMode' in config)
-
     def test_create_host_config_no_options_newer_api_version(self):
-        config = create_host_config(version='1.20')
-        self.assertEqual(config['NetworkMode'], 'default')
+        config = create_host_config(version='1.21')
+        assert config['NetworkMode'] == 'default'
 
     def test_create_host_config_invalid_cpu_cfs_types(self):
         with pytest.raises(TypeError):
-            create_host_config(version='1.20', cpu_quota='0')
+            create_host_config(version='1.21', cpu_quota='0')
 
         with pytest.raises(TypeError):
-            create_host_config(version='1.20', cpu_period='0')
+            create_host_config(version='1.21', cpu_period='0')
 
         with pytest.raises(TypeError):
-            create_host_config(version='1.20', cpu_quota=23.11)
+            create_host_config(version='1.21', cpu_quota=23.11)
 
         with pytest.raises(TypeError):
-            create_host_config(version='1.20', cpu_period=1999.0)
+            create_host_config(version='1.21', cpu_period=1999.0)
 
     def test_create_host_config_with_cpu_quota(self):
-        config = create_host_config(version='1.20', cpu_quota=1999)
-        self.assertEqual(config.get('CpuQuota'), 1999)
+        config = create_host_config(version='1.21', cpu_quota=1999)
+        assert config.get('CpuQuota') == 1999
 
     def test_create_host_config_with_cpu_period(self):
-        config = create_host_config(version='1.20', cpu_period=1999)
-        self.assertEqual(config.get('CpuPeriod'), 1999)
+        config = create_host_config(version='1.21', cpu_period=1999)
+        assert config.get('CpuPeriod') == 1999
 
     def test_create_host_config_with_blkio_constraints(self):
         blkio_rate = [{"Path": "/dev/sda", "Rate": 1000}]
-        config = create_host_config(version='1.22',
-                                    blkio_weight=1999,
-                                    blkio_weight_device=blkio_rate,
-                                    device_read_bps=blkio_rate,
-                                    device_write_bps=blkio_rate,
-                                    device_read_iops=blkio_rate,
-                                    device_write_iops=blkio_rate)
+        config = create_host_config(
+            version='1.22', blkio_weight=1999, blkio_weight_device=blkio_rate,
+            device_read_bps=blkio_rate, device_write_bps=blkio_rate,
+            device_read_iops=blkio_rate, device_write_iops=blkio_rate
+        )
 
-        self.assertEqual(config.get('BlkioWeight'), 1999)
-        self.assertTrue(config.get('BlkioWeightDevice') is blkio_rate)
-        self.assertTrue(config.get('BlkioDeviceReadBps') is blkio_rate)
-        self.assertTrue(config.get('BlkioDeviceWriteBps') is blkio_rate)
-        self.assertTrue(config.get('BlkioDeviceReadIOps') is blkio_rate)
-        self.assertTrue(config.get('BlkioDeviceWriteIOps') is blkio_rate)
-        self.assertEqual(blkio_rate[0]['Path'], "/dev/sda")
-        self.assertEqual(blkio_rate[0]['Rate'], 1000)
+        assert config.get('BlkioWeight') == 1999
+        assert config.get('BlkioWeightDevice') is blkio_rate
+        assert config.get('BlkioDeviceReadBps') is blkio_rate
+        assert config.get('BlkioDeviceWriteBps') is blkio_rate
+        assert config.get('BlkioDeviceReadIOps') is blkio_rate
+        assert config.get('BlkioDeviceWriteIOps') is blkio_rate
+        assert blkio_rate[0]['Path'] == "/dev/sda"
+        assert blkio_rate[0]['Rate'] == 1000
 
     def test_create_host_config_with_shm_size(self):
         config = create_host_config(version='1.22', shm_size=67108864)
-        self.assertEqual(config.get('ShmSize'), 67108864)
+        assert config.get('ShmSize') == 67108864
 
     def test_create_host_config_with_shm_size_in_mb(self):
         config = create_host_config(version='1.22', shm_size='64M')
-        self.assertEqual(config.get('ShmSize'), 67108864)
+        assert config.get('ShmSize') == 67108864
 
     def test_create_host_config_with_oom_kill_disable(self):
-        config = create_host_config(version='1.20', oom_kill_disable=True)
-        self.assertEqual(config.get('OomKillDisable'), True)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(version='1.18.3',
-                                                       oom_kill_disable=True))
+        config = create_host_config(version='1.21', oom_kill_disable=True)
+        assert config.get('OomKillDisable') is True
 
     def test_create_host_config_with_userns_mode(self):
         config = create_host_config(version='1.23', userns_mode='host')
-        self.assertEqual(config.get('UsernsMode'), 'host')
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(version='1.22',
-                                                       userns_mode='host'))
-        self.assertRaises(
-            ValueError, lambda: create_host_config(version='1.23',
-                                                   userns_mode='host12'))
+        assert config.get('UsernsMode') == 'host'
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.22', userns_mode='host')
+        with pytest.raises(ValueError):
+            create_host_config(version='1.23', userns_mode='host12')
 
     def test_create_host_config_with_oom_score_adj(self):
         config = create_host_config(version='1.22', oom_score_adj=100)
-        self.assertEqual(config.get('OomScoreAdj'), 100)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(version='1.21',
-                                                       oom_score_adj=100))
-        self.assertRaises(
-            TypeError, lambda: create_host_config(version='1.22',
-                                                  oom_score_adj='100'))
+        assert config.get('OomScoreAdj') == 100
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.21', oom_score_adj=100)
+        with pytest.raises(TypeError):
+            create_host_config(version='1.22', oom_score_adj='100')
 
     def test_create_host_config_with_dns_opt(self):
 
@@ -112,30 +99,20 @@ class HostConfigTest(unittest.TestCase):
         config = create_host_config(version='1.21', dns_opt=tested_opts)
         dns_opts = config.get('DnsOptions')
 
-        self.assertTrue('use-vc' in dns_opts)
-        self.assertTrue('no-tld-query' in dns_opts)
-
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(version='1.20',
-                                                       dns_opt=tested_opts))
+        assert 'use-vc' in dns_opts
+        assert 'no-tld-query' in dns_opts
 
     def test_create_host_config_with_mem_reservation(self):
         config = create_host_config(version='1.21', mem_reservation=67108864)
-        self.assertEqual(config.get('MemoryReservation'), 67108864)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(
-                version='1.20', mem_reservation=67108864))
+        assert config.get('MemoryReservation') == 67108864
 
     def test_create_host_config_with_kernel_memory(self):
         config = create_host_config(version='1.21', kernel_memory=67108864)
-        self.assertEqual(config.get('KernelMemory'), 67108864)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(
-                version='1.20', kernel_memory=67108864))
+        assert config.get('KernelMemory') == 67108864
 
     def test_create_host_config_with_pids_limit(self):
         config = create_host_config(version='1.23', pids_limit=1024)
-        self.assertEqual(config.get('PidsLimit'), 1024)
+        assert config.get('PidsLimit') == 1024
 
         with pytest.raises(InvalidVersion):
             create_host_config(version='1.22', pids_limit=1024)
@@ -144,7 +121,7 @@ class HostConfigTest(unittest.TestCase):
 
     def test_create_host_config_with_isolation(self):
         config = create_host_config(version='1.24', isolation='hyperv')
-        self.assertEqual(config.get('Isolation'), 'hyperv')
+        assert config.get('Isolation') == 'hyperv'
 
         with pytest.raises(InvalidVersion):
             create_host_config(version='1.23', isolation='hyperv')
@@ -167,9 +144,6 @@ class HostConfigTest(unittest.TestCase):
             create_host_config(version='1.24', mem_swappiness='40')
 
     def test_create_host_config_with_volume_driver(self):
-        with pytest.raises(InvalidVersion):
-            create_host_config(version='1.20', volume_driver='local')
-
         config = create_host_config(version='1.21', volume_driver='local')
         assert config.get('VolumeDriver') == 'local'
 
@@ -179,10 +153,9 @@ class HostConfigTest(unittest.TestCase):
 
     def test_create_host_config_with_cpu_count(self):
         config = create_host_config(version='1.25', cpu_count=2)
-        self.assertEqual(config.get('CpuCount'), 2)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(
-                version='1.24', cpu_count=1))
+        assert config.get('CpuCount') == 2
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.24', cpu_count=1)
 
     def test_create_host_config_invalid_cpu_percent_types(self):
         with pytest.raises(TypeError):
@@ -190,10 +163,9 @@ class HostConfigTest(unittest.TestCase):
 
     def test_create_host_config_with_cpu_percent(self):
         config = create_host_config(version='1.25', cpu_percent=15)
-        self.assertEqual(config.get('CpuPercent'), 15)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(
-                version='1.24', cpu_percent=10))
+        assert config.get('CpuPercent') == 15
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.24', cpu_percent=10)
 
     def test_create_host_config_invalid_nano_cpus_types(self):
         with pytest.raises(TypeError):
@@ -201,23 +173,29 @@ class HostConfigTest(unittest.TestCase):
 
     def test_create_host_config_with_nano_cpus(self):
         config = create_host_config(version='1.25', nano_cpus=1000)
-        self.assertEqual(config.get('NanoCpus'), 1000)
-        self.assertRaises(
-            InvalidVersion, lambda: create_host_config(
-                version='1.24', nano_cpus=1))
+        assert config.get('NanoCpus') == 1000
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.24', nano_cpus=1)
 
+    def test_create_host_config_with_cpu_rt_period_types(self):
+        with pytest.raises(TypeError):
+            create_host_config(version='1.25', cpu_rt_period='1000')
 
-class ContainerConfigTest(unittest.TestCase):
-    def test_create_container_config_volume_driver_warning(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter('always')
-            ContainerConfig(
-                version='1.21', image='scratch', command=None,
-                volume_driver='local'
-            )
+    def test_create_host_config_with_cpu_rt_period(self):
+        config = create_host_config(version='1.25', cpu_rt_period=1000)
+        assert config.get('CPURealtimePeriod') == 1000
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.24', cpu_rt_period=1000)
 
-        assert len(w) == 1
-        assert 'The volume_driver option has been moved' in str(w[0].message)
+    def test_ctrate_host_config_with_cpu_rt_runtime_types(self):
+        with pytest.raises(TypeError):
+            create_host_config(version='1.25', cpu_rt_runtime='1000')
+
+    def test_create_host_config_with_cpu_rt_runtime(self):
+        config = create_host_config(version='1.25', cpu_rt_runtime=1000)
+        assert config.get('CPURealtimeRuntime') == 1000
+        with pytest.raises(InvalidVersion):
+            create_host_config(version='1.24', cpu_rt_runtime=1000)
 
 
 class ContainerSpecTest(unittest.TestCase):
@@ -242,43 +220,46 @@ class UlimitTest(unittest.TestCase):
         config = create_host_config(
             ulimits=[ulimit_dct], version=DEFAULT_DOCKER_API_VERSION
         )
-        self.assertIn('Ulimits', config)
-        self.assertEqual(len(config['Ulimits']), 1)
+        assert 'Ulimits' in config
+        assert len(config['Ulimits']) == 1
         ulimit_obj = config['Ulimits'][0]
-        self.assertTrue(isinstance(ulimit_obj, Ulimit))
-        self.assertEqual(ulimit_obj.name, ulimit_dct['name'])
-        self.assertEqual(ulimit_obj.soft, ulimit_dct['soft'])
-        self.assertEqual(ulimit_obj['Soft'], ulimit_obj.soft)
+        assert isinstance(ulimit_obj, Ulimit)
+        assert ulimit_obj.name == ulimit_dct['name']
+        assert ulimit_obj.soft == ulimit_dct['soft']
+        assert ulimit_obj['Soft'] == ulimit_obj.soft
 
     def test_create_host_config_dict_ulimit_capitals(self):
         ulimit_dct = {'Name': 'nofile', 'Soft': 8096, 'Hard': 8096 * 4}
         config = create_host_config(
             ulimits=[ulimit_dct], version=DEFAULT_DOCKER_API_VERSION
         )
-        self.assertIn('Ulimits', config)
-        self.assertEqual(len(config['Ulimits']), 1)
+        assert 'Ulimits' in config
+        assert len(config['Ulimits']) == 1
         ulimit_obj = config['Ulimits'][0]
-        self.assertTrue(isinstance(ulimit_obj, Ulimit))
-        self.assertEqual(ulimit_obj.name, ulimit_dct['Name'])
-        self.assertEqual(ulimit_obj.soft, ulimit_dct['Soft'])
-        self.assertEqual(ulimit_obj.hard, ulimit_dct['Hard'])
-        self.assertEqual(ulimit_obj['Soft'], ulimit_obj.soft)
+        assert isinstance(ulimit_obj, Ulimit)
+        assert ulimit_obj.name == ulimit_dct['Name']
+        assert ulimit_obj.soft == ulimit_dct['Soft']
+        assert ulimit_obj.hard == ulimit_dct['Hard']
+        assert ulimit_obj['Soft'] == ulimit_obj.soft
 
     def test_create_host_config_obj_ulimit(self):
         ulimit_dct = Ulimit(name='nofile', soft=8096)
         config = create_host_config(
             ulimits=[ulimit_dct], version=DEFAULT_DOCKER_API_VERSION
         )
-        self.assertIn('Ulimits', config)
-        self.assertEqual(len(config['Ulimits']), 1)
+        assert 'Ulimits' in config
+        assert len(config['Ulimits']) == 1
         ulimit_obj = config['Ulimits'][0]
-        self.assertTrue(isinstance(ulimit_obj, Ulimit))
-        self.assertEqual(ulimit_obj, ulimit_dct)
+        assert isinstance(ulimit_obj, Ulimit)
+        assert ulimit_obj == ulimit_dct
 
     def test_ulimit_invalid_type(self):
-        self.assertRaises(ValueError, lambda: Ulimit(name=None))
-        self.assertRaises(ValueError, lambda: Ulimit(name='hello', soft='123'))
-        self.assertRaises(ValueError, lambda: Ulimit(name='hello', hard='456'))
+        with pytest.raises(ValueError):
+            Ulimit(name=None)
+        with pytest.raises(ValueError):
+            Ulimit(name='hello', soft='123')
+        with pytest.raises(ValueError):
+            Ulimit(name='hello', hard='456')
 
 
 class LogConfigTest(unittest.TestCase):
@@ -287,18 +268,18 @@ class LogConfigTest(unittest.TestCase):
         config = create_host_config(
             version=DEFAULT_DOCKER_API_VERSION, log_config=dct
         )
-        self.assertIn('LogConfig', config)
-        self.assertTrue(isinstance(config['LogConfig'], LogConfig))
-        self.assertEqual(dct['type'], config['LogConfig'].type)
+        assert 'LogConfig' in config
+        assert isinstance(config['LogConfig'], LogConfig)
+        assert dct['type'] == config['LogConfig'].type
 
     def test_create_host_config_obj_logconfig(self):
         obj = LogConfig(type=LogConfig.types.SYSLOG, config={'key1': 'val1'})
         config = create_host_config(
             version=DEFAULT_DOCKER_API_VERSION, log_config=obj
         )
-        self.assertIn('LogConfig', config)
-        self.assertTrue(isinstance(config['LogConfig'], LogConfig))
-        self.assertEqual(obj, config['LogConfig'])
+        assert 'LogConfig' in config
+        assert isinstance(config['LogConfig'], LogConfig)
+        assert obj == config['LogConfig']
 
     def test_logconfig_invalid_config_type(self):
         with pytest.raises(ValueError):
@@ -320,7 +301,7 @@ class IPAMConfigTest(unittest.TestCase):
                              gateway='192.168.52.254')
 
         ipam_config = IPAMConfig(pool_configs=[ipam_pool])
-        self.assertEqual(ipam_config, {
+        assert ipam_config == {
             'Driver': 'default',
             'Config': [{
                 'Subnet': '192.168.52.0/24',
@@ -328,7 +309,7 @@ class IPAMConfigTest(unittest.TestCase):
                 'AuxiliaryAddresses': None,
                 'IPRange': None,
             }]
-        })
+        }
 
 
 class ServiceModeTest(unittest.TestCase):
@@ -413,3 +394,77 @@ class MountTest(unittest.TestCase):
         assert mount['Source'] == "C:/foo/bar"
         assert mount['Target'] == "/baz"
         assert mount['Type'] == 'bind'
+
+
+class ServicePortsTest(unittest.TestCase):
+    def test_convert_service_ports_simple(self):
+        ports = {8080: 80}
+        assert convert_service_ports(ports) == [{
+            'Protocol': 'tcp',
+            'PublishedPort': 8080,
+            'TargetPort': 80,
+        }]
+
+    def test_convert_service_ports_with_protocol(self):
+        ports = {8080: (80, 'udp')}
+
+        assert convert_service_ports(ports) == [{
+            'Protocol': 'udp',
+            'PublishedPort': 8080,
+            'TargetPort': 80,
+        }]
+
+    def test_convert_service_ports_with_protocol_and_mode(self):
+        ports = {8080: (80, 'udp', 'ingress')}
+
+        assert convert_service_ports(ports) == [{
+            'Protocol': 'udp',
+            'PublishedPort': 8080,
+            'TargetPort': 80,
+            'PublishMode': 'ingress',
+        }]
+
+    def test_convert_service_ports_invalid(self):
+        ports = {8080: ('way', 'too', 'many', 'items', 'here')}
+
+        with pytest.raises(ValueError):
+            convert_service_ports(ports)
+
+    def test_convert_service_ports_no_protocol_and_mode(self):
+        ports = {8080: (80, None, 'host')}
+
+        assert convert_service_ports(ports) == [{
+            'Protocol': 'tcp',
+            'PublishedPort': 8080,
+            'TargetPort': 80,
+            'PublishMode': 'host',
+        }]
+
+    def test_convert_service_ports_multiple(self):
+        ports = {
+            8080: (80, None, 'host'),
+            9999: 99,
+            2375: (2375,)
+        }
+
+        converted_ports = convert_service_ports(ports)
+        assert {
+            'Protocol': 'tcp',
+            'PublishedPort': 8080,
+            'TargetPort': 80,
+            'PublishMode': 'host',
+        } in converted_ports
+
+        assert {
+            'Protocol': 'tcp',
+            'PublishedPort': 9999,
+            'TargetPort': 99,
+        } in converted_ports
+
+        assert {
+            'Protocol': 'tcp',
+            'PublishedPort': 2375,
+            'TargetPort': 2375,
+        } in converted_ports
+
+        assert len(converted_ports) == 3
